import { computed, defineComponent, onMounted, reactive, ref } from 'vue';
import styles from './detail.module.less';
import CBreadcrumb from '/src/components/CBreadcrumb';
import { useRoute } from 'vue-router';
import { NForm, NFormItem, NScrollbar, NSpin, NTabPane, NTabs } from 'naive-ui';
import SearchGroupResources from './search-group-resources';
import TheSearch from '/src/components/TheSearch';
import TheEmpty from '/src/components/TheEmpty';
import { audioPlayType } from '/src/utils/contants';
import { api_musicSheetPage } from '../xiaoku-ai/api';
import { api_musicSheetTagQueryList } from './api';
import { formatUsedNum } from '.';

export default defineComponent({
  name: 'xiaoku-detail',
  setup() {
    const route = useRoute();
    const routerList = ref([
      { name: 'AI学练', path: '/xiaoku-list' },
      { name: '全部曲目', path: '' }
    ]);
    const forms = reactive({
      page: 1,
      rows: 24,
      status: true,
      searchType: '',
    });
    const state = reactive({
      loading: true,
      finshed: false,
      reshing: false,
      tabName: '',
      list: [] as any,
      allSearch: {
        name: '',
        musicTagIds: '',
        audioPlayTypes: null as any,
        bookVersionId: null as any,
        subjectId: null
      },
      hotSearch: {
        name: '',
      },
      newSearch: {
        name: '',
      },
      recommendSearch: {
        name: '',
      }
    });

    const searchValue = computed(() => {
      if(state.tabName === 'RECOMMEND') {
        return state.recommendSearch.name
      } else if(state.tabName === 'HOT') {
        return state.hotSearch.name
      } else if(state.tabName === 'NEW') {
        return state.newSearch.name
      } else {
        return state.allSearch.name
      }
    })

    const onSearch = async (item: any) => {
      forms.page = 1;
      state.reshing = true;
      state.finshed = false;

      const { subjectId, ...res } = item;
      state.allSearch = Object.assign(state.allSearch, {
        ...res,
        musicalInstrumentId: subjectId,
        subjectId: null
      });

      getList();
    };


    const spinRef = ref();
    const handleResh = () => {
      if (state.loading || state.finshed) return;
      forms.page = forms.page + 1;
      getList();
    };

    const getList = async () => {
      state.loading = true;
      let res = {} as any;
      try {
        const { ...result } = forms;
        let params = {
          ...result,
          searchType: state.tabName
        } as any
        if(state.tabName === 'RECOMMEND') {
          params = Object.assign(params, state.recommendSearch);
        } else if(state.tabName === 'HOT') {
          params = Object.assign(params, state.hotSearch);
        } else if(state.tabName === 'NEW') {
          params = Object.assign(params, state.newSearch);
        } else {
          params.name = state.allSearch.name
          const { audioPlayTypes, ...more } = state.allSearch
          params = Object.assign(params, { ...more });
          params.audioPlayTypes =  audioPlayTypes
            ? audioPlayTypes === 'PLAY_SING'
              ? ['PLAY', 'SING']
              : [audioPlayTypes]
            : []
        }
        res = await api_musicSheetPage(params);
      } catch (error) {
        console.log(error);
      }
      if (state.reshing) {
        state.list = [];
        state.reshing = false;
      }

      if (res?.code === 200 && Array.isArray(res?.data?.rows)) {
        const tempResult = res?.data?.rows || [];
        tempResult.forEach((item: any) => {
          item.audioPlayTypeArray = item.audioPlayTypes
            ? item.audioPlayTypes.split(',')
            : [];
        });
        state.list = [...state.list, ...res.data.rows];
        state.finshed = forms.page >= res.data.pages; 
      } else {
        state.finshed = true;
      }
      state.loading = false;
    };

    const __initSpin = () => {
      const obv = new IntersectionObserver(entries => {
        if (entries[0].intersectionRatio > 0) {
          handleResh();
        }
      });
      obv.observe(spinRef.value);
    }

    onMounted(async () => {
      // getList();
      __initSpin()
    });
    return () => (
      <div class={styles.xiaokuDetail}>
        <CBreadcrumb list={routerList.value}></CBreadcrumb>
        <div class={styles.detailContainer}>
          <NTabs
            paneClass={styles.paneTitle}
            justifyContent="start"
            // animated
            paneWrapperClass={styles.paneWrapperContainer}
            v-model:value={state.tabName}
            onUpdate:value={(val: any) => {
              forms.page = 1;
              state.finshed = false;
              state.reshing = true;
              state.list = []
              getList();

              __initSpin()
            }}
            v-slots={{
              suffix: () => (
                <TheSearch
                  placeholder='请输入曲目名称'
                  round
                  value={searchValue.value}
                  class={styles.inputSearch}
                  onSearch={val => {
                    if(state.tabName === 'RECOMMEND') {
                      state.recommendSearch.name = val
                    } else if(state.tabName === 'HOT') {
                      state.hotSearch.name = val
                    } else if(state.tabName === 'NEW') {
                      state.newSearch.name = val
                    } else {
                      state.allSearch.name = val
                    }
                    forms.page = 1;
                    state.finshed = false;
                    state.list = []
                    getList();
                  }}
                />
              )
            }}>
            <NTabPane name={``} tab={'全部曲目'}></NTabPane>
            <NTabPane name={`RECOMMEND`} tab={'推荐曲目'}></NTabPane>
            <NTabPane name={`HOT`} tab={'热门曲目'}></NTabPane>
            <NTabPane name={`NEW`} tab={'最新曲目'}></NTabPane>
          </NTabs>

          <NScrollbar class={[[styles.wrapList, !state.loading && state.list.length === 0 && styles.wrapListEmpty]]}>
              <div class={[styles.searchSection, state.tabName ? styles.searchSectionHide : '']}>
                <SearchGroupResources onSearch={(val: any) => {
                  onSearch(val)
                }} />
              </div>
            
            {state.list.length > 0 && <div
              class={[
                styles.sectionContainer,
                state.tabName && styles.noSearchContainer
              ]}>
              {state.list.map((item: any) => (
                <div class={styles.sectionItem}>
                  <div class={styles.img}>
                    <img referrerpolicy="no-referrer" src={item.titleImg} />
                  </div>
                  <div class={styles.infos}>
                    <div class={styles.topName}>{item.musicSheetName}</div>
                    <div class={styles.types}>
                      <div class={styles.hot}>
                        <span>{formatUsedNum(item.usedNum)}</span>
                      </div>
                      {item.audioPlayTypes?.includes('SING') && (
                        <div class={styles.sing}>演唱</div>
                      )}
                      {item.audioPlayTypes?.includes('PLAY') && (
                        <div class={styles.song}>演奏</div>
                      )}

                      <div class={styles.author}>{item.composer}</div>
                    </div>
                  </div>
                </div>
              ))}
            </div>}
            

            <div
              ref={spinRef}
              class={[styles.loadingWrap, state.finshed && styles.showLoading]}>
              <NSpin show={true}></NSpin>
            </div>

            {!state.loading && state.list.length === 0 && (
              <div class={styles.empty}>
                <TheEmpty></TheEmpty>
              </div>
            )}
          </NScrollbar>
        </div>
      </div>
    );
  }
});
